addonName, L = ...;

-- DEFAULT SETTINGS:
DEFAULT_SETTINGS = 
{
	["NamePlates/ShowClassIcon"] = true,
	["NamePlates/BorderColourMode"] = "team",
	["NamePlates/HealthBarColourMode"] = "class",
}

-- Create a new Handler:
local NamePlate = ArenaLiveCore:AddHandler("NamePlate", "EventCore");
local NameText = ArenaLiveCore:GetHandler("NameText");

-- Register for some events to track unitIDs:
NamePlate:RegisterEvent("GROUP_ROSTER_UPDATE");
NamePlate:RegisterEvent("ARENA_OPPONENT_UPDATE");
NamePlate:RegisterEvent("ADDON_LOADED");

-- Max units of a certain type:
local MAX_RAID_MEMBERS = 40;
local MAX_PARTY_MEMBERS = 4;
local MAX_ARENA_OPPONENTS = 5;

-- Table to store unitIDs:
NamePlate.UnitCache = {};
NamePlate.NameCache = {};

-- Table to store all newly created name plates with reference to the ones that they replace:
NamePlate.NamePlates = {};

-- Number to store currently number of children of WorldFrame:
NamePlate.NumWorldFrameChildren = 0;

local function NamePlate_Update(namePlate)

	-- Update UnitCache:
	NamePlate:UpdateUnitCache("UPDATE_ALL", nil, nil, false);

	-- Crawl data from Blizzard's NamePlate:
	local healthBarValue = namePlate.parent.healthBar:GetValue();
	local healthBarMin, healthbarMax = namePlate.parent.healthBar:GetMinMaxValues();
	local name = namePlate.parent.nameText:GetText();
	
	-- Set new healthbar Values:
	namePlate.healthBar:SetMinMaxValues(healthBarMin, healthbarMax);
	namePlate.healthBar:SetValue(healthBarValue);
	namePlate.healthBar.minValue, namePlate.healthBar.maxValue = healthBarMin, healthbarMax;
	namePlate.healthBar.currValue = healthBarValue;

	-- Reset old unit information:
	namePlate.unit = nil;
	namePlate.guid = nil;
	namePlate.class = nil;
	namePlate.reaction = nil;
	
	-- Now check if we can get unit information:
	local nameAlias;
	local plateReaction = NamePlate:GetReactionType(namePlate.parent.healthBar:GetStatusBarColor());
	namePlate.reaction = plateReaction;
	if ( NamePlate.NameCache[name] ) then
		local unit = NamePlate.NameCache[name];
		local isPlayer = UnitIsPlayer(unit);
		local unitReaction = NamePlate:GetReactionType(UnitSelectionColor(unit));
		
		if ( NamePlate:ReactionIsReaction(unitReaction, isPlayer, plateReaction) ) then
			-- It is relatively sure from this point on that the shown name plate shows this particular unitID:
			namePlate.unit = unit;
			namePlate.guid = UnitGUID(unit);
			
			-- Get class information:
			if ( isPlayer ) then
				local _, class = UnitClass(unit);
				_, class = UnitClass(unit);
				namePlate.class = class;
			end
				
			-- Check if there is a name alias:
			local _, realm = UnitName(unit);
			if ( not realm or realm == "" ) then
				realm = GetRealmName();
			end
				
			nameAlias = NameText:GetNameAlias(unit);
		end
	end

	-- Set new name:
	if ( nameAlias ) then
		namePlate.nameText:SetText(nameAlias);
	else
		namePlate.nameText:SetText(name);
	end
	
	NamePlate:StyleNamePlate(namePlate)
end

local function NamePlate_UpdateCastBar(castBar)
	
	local plate = castBar:GetParent();
	
	-- Update Min and Max values:
	local minValue, maxValue = plate.parent.castBar:GetMinMaxValues();
	castBar:SetMinMaxValues(minValue, maxValue);
	castBar.minValue = minValue;
	castBar.maxValue = maxValue;
	
	-- Update current Value:
	local value = plate.parent.castBar:GetValue();
	castBar:SetValue(value);
	castBar.value = value;
	
	-- Update Icon:
	local iconTexture = plate.parent.castBar.icon:GetTexture();
	castBar.icon:SetTexture(iconTexture);

	-- Update Text:
	local text = plate.parent.castBar.text:GetText();
	castBar.text:SetText(text);
end

local function NamePlate_HealthBar_OnValueChanged(blizzHealthBar)
	local blizzPlate = blizzHealthBar:GetParent():GetParent();
	local affectedPlate = NamePlate.NamePlates[blizzPlate];
	local value = blizzHealthBar:GetValue();
	
	affectedPlate.healthBar:SetValue(value);
	affectedPlate.healthBar.value = value;
end

local function NamePlate_HealthBar_OnMinMaxChanged(blizzHealthBar)
	local blizzPlate = blizzHealthBar:GetParent():GetParent();
	local affectedPlate = NamePlate.NamePlates[blizzPlate];
	local minValue, maxValue = blizzHealthBar:GetMinMaxValues();
	
	affectedPlate.healthBar:SetMinMaxValues(minValue, maxValue);
end

local function NamePlate_CastBar_OnValueChanged(blizzCastBar)
	local blizzPlate = blizzCastBar:GetParent():GetParent();
	local affectedPlate = NamePlate.NamePlates[blizzPlate];
	local value = blizzCastBar:GetValue();
	affectedPlate.castBar:SetValue(value);
	affectedPlate.castBar.value = castBarValue;
end

local function NamePlate_CastBar_OnMinMaxChanged(blizzCastBar)
	local blizzPlate = blizzCastBar:GetParent():GetParent();
	local affectedPlate = NamePlate.NamePlates[blizzPlate];
	local minValue, maxValue = blizzCastBar:GetMinMaxValues();
	NamePlate_UpdateCastBar(affectedPlate.castBar);
end

local function NamePlate_CastBar_OnShow(blizzCastBar)
	local blizzPlate = blizzCastBar:GetParent():GetParent();
	local affectedPlate = NamePlate.NamePlates[blizzPlate];
	affectedPlate.castBar:Show();
end

local function NamePlate_CastBar_OnHide(blizzCastBar)
	local blizzPlate = blizzCastBar:GetParent():GetParent();
	local affectedPlate = NamePlate.NamePlates[blizzPlate];	
	affectedPlate.castBar:Hide();
end
local function NamePlate_OnUpdate(self, elapsed)
end

--[[
	What functions do the nameplates need?
		CORE FUNCTIONS:
		- NamePlates will only be replaced inside the arena.
		- The Handler must be able to find out wether the nameplate shows a unit that has a unitID or not.
			->  If it has a unitID, use Unit-Functions to update the Nameplate.
			-> If not, use an OnUpdate function that constantly gets the value of the original nameplate.
		- The only way to get the number of NamePlates is to get all children of the WorldFrame, since WorldFrame is parent to all NamePlates.
		- The Handler must store all original nameplates in a table with a reference to the nameplate that replaces it.
			-> Simply try to keep these always together and just change the unit information that the replacing nameplate shows.
		- Set the original nameplate as a parent to the new nameplate. This way the nameplate is automatically shown and hidden with the nameplate it replaces.
			-> The other child elements of the nameplate need to be invisible so that only the ArenaLiveSpectator nameplate is shown. Alpha for the mainChild and nameChild should be enough.
			   Alpha is not affected by CombatLockDown. (Hiding/Showing them instead could cause errors when in combat lockdown, but I need to test that).
		
		STYLE FUNCTIONS:
		- The Frame Border needs to be coloured in the team's colour to differentiate between teams easier.
		- Class Icons should be shown for every participant.
		- Further Unit data (e.g. Absorb, predict healing etc.) can be shown for all nameplates that show a unit with a unitID.
]]--
local mainFrame, nameFrame, nameFontString, healthBar, castBar;
function NamePlate:SetNamePlateStructure(blizzPlate)

	-- Get 2 main frames of a name plate:
	mainFrame, nameFrame = blizzPlate:GetChildren();
	blizzPlate.mainFrame = mainFrame;
	blizzPlate.nameFrame = nameFrame;
	
	-- Get castbar and healthbar of a nameplate:
	healthBar = select(1, mainFrame:GetChildren());
	castBar = select(2, mainFrame:GetChildren());
	blizzPlate.healthBar = healthBar;
	blizzPlate.castBar = castBar;
	blizzPlate.castBar = castBar;
	blizzPlate.castBar.icon = select(4, castBar:GetRegions());
	blizzPlate.castBar.text = select(5, castBar:GetRegions());
	
	-- Get Name FontString:
	nameFontString = select(1, nameFrame:GetRegions());
	blizzPlate.nameText = nameFontString;

end

function NamePlate:HideBlizzardNamePlate(blizzPlate)
	-- Hiding the both most on top frames should be enough:
	blizzPlate.mainFrame:SetAlpha(0);
	blizzPlate.nameFrame:SetAlpha(0);
end

function NamePlate:CreateNamePlate(blizzPlate)
	
	-- Create the new frame:
	local index = string.match(blizzPlate:GetName(), "^NamePlate(%d*)$");
	local newPlate = CreateFrame("Frame", "ALSpec_NamePlate"..index, blizzPlate, "ArenaLive_NamePlateTemplate");
	local prefix = newPlate:GetName();
	NamePlate.NamePlates[blizzPlate] = newPlate;
	
	-- Reference to the original name plate:
	newPlate.parent = blizzPlate;

	-- References inside the frame:
	newPlate.healthBar = _G[prefix.."HealthBar"];
	newPlate.border = _G[prefix.."Border"];
	newPlate.nameText = _G[prefix.."NameText"];
	newPlate.classIcon = _G[prefix.."ClassIcon"];
	newPlate.castBar = _G[prefix.."CastBar"];
	newPlate.castBar.icon = _G[prefix.."CastBarIcon"];
	newPlate.castBar.text = _G[prefix.."CastBarText"];
	
	-- Set Script Handlers:
	blizzPlate.healthBar:HookScript("OnValueChanged", NamePlate_HealthBar_OnValueChanged);
	blizzPlate.healthBar:HookScript("OnMinMaxChanged", NamePlate_HealthBar_OnMinMaxChanged);
	blizzPlate.castBar:HookScript("OnValueChanged", NamePlate_CastBar_OnValueChanged);
	blizzPlate.castBar:HookScript("OnMinMaxChanged", NamePlate_CastBar_OnMinMaxChanged);
	blizzPlate.castBar:HookScript("OnShow", NamePlate_CastBar_OnShow);
	blizzPlate.castBar:HookScript("OnHide", NamePlate_CastBar_OnHide);
	
	--newPlate:SetScript("OnUpdate", NamePlate_OnUpdate);
	newPlate:SetScript("OnShow", NamePlate_Update);
	newPlate.castBar:SetScript("OnShow", NamePlate_UpdateCastBar);
	-- Hide its parent Plate:
	NamePlate:HideBlizzardNamePlate(blizzPlate);

	-- Update Initially: 
	NamePlate_Update(newPlate);
	
end

function NamePlate:GetTeamColour(unit, reaction)

	-- Get team colours if needed:
	local _, instanceType = IsInInstance();
	local red, green, blue;
	if ( unit and IsAddOnLoaded("ArenaLiveSpectator") ) then 
	
		local match;
		for i = 1, 3 do
			local checkName = ArenaLiveCore:GetDBEntry("ArenaLiveSpectator", "Teams/Left/Member"..i);
			local ownName = ArenaLiveSpectator:GetUnitName(unit);
			if ( checkName == ownName ) then
				match = "Left";
				break;
			end
				
			checkName = ArenaLiveCore:GetDBEntry("ArenaLiveSpectator", "Teams/Right/Member"..i);
			if ( checkName == ownName ) then
				match = "Right";
				break;
			end
		end
		
		if ( match ) then
			red, green, blue = unpack(ArenaLiveSpectator.teamColours[match]);
		end
	elseif ( instanceType == "arena" and IsAddOnLoaded("ArenaLiveSpectator") ) then
		-- Colour Pet name plates in team colour during arena:
		if ( reaction == "PvP-Friendly" ) then
			red, green, blue = unpack(ArenaLiveSpectator.teamColours["Left"]);
		elseif ( reaction == "Hostile" ) then
			red, green, blue = unpack(ArenaLiveSpectator.teamColours["Right"]);
		end	
	
	end
	
	return red, green, blue;
end

function NamePlate:GetReactionColour(reaction)
	
	local red, green, blue;

	if ( reaction == "Friendly" ) then
		red, green, blue = 0, 0, 1;
	elseif ( reaction == "PvP-Friendly" ) then
		red, green, blue = 0, 1, 0;
	elseif ( reaction == "Hostile" or reaction == "Hostile-Class" ) then
		red, green, blue = 1, 0, 0;
	elseif ( reaction == "Neutral" ) then
		red, green, blue = 1, 1, 0;
	end
	
	return red, green, blue;
end

function NamePlate:GetClassColour(blizzPlate, class, reaction)
	
	local red, green, blue;
	if ( class ) then
		red, green, blue = RAID_CLASS_COLORS[class]["r"], RAID_CLASS_COLORS[class]["g"], RAID_CLASS_COLORS[class]["b"];
	elseif ( reaction == "Hostile-Class" ) then
		red, green, blue = blizzPlate.healthBar:GetStatusBarColor();
	end

	return red, green, blue;
end

function NamePlate:UpdateClassIcon(namePlate)

	local class = namePlate.class;
	local unit = namePlate.unit;
	local showClassIcon = ArenaLiveCore:GetDBEntry(addonName, "NamePlates/ShowClassIcon");

	
	if ( unit and class and showClassIcon ) then
		namePlate.classIcon:Show();
		namePlate.classIcon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]));
	else
		namePlate.classIcon:Hide();
	end	
	
end

function NamePlate:UpdateHealthbarColour(namePlate)
	local class = namePlate.class;
	local unit = namePlate.unit;
	local reaction = namePlate.reaction;
	local red, green, blue;
	local colourMode = ArenaLiveCore:GetDBEntry(addonName, "NamePlates/HealthBarColourMode");
	
	if ( colourMode == "class" and ( class or reaction == "Hostile-Class" ) ) then
		red, green, blue = NamePlate:GetClassColour(namePlate.parent, class, reaction);
	elseif ( colourMode == "team" and IsAddOnLoaded("ArenaLiveSpectator") ) then	
		red, green, blue = NamePlate:GetTeamColour(unit, reaction);
		if ( not red ) then
			red, green, blue = NamePlate:GetReactionColour(reaction);
		end
	else
		red, green, blue = namePlate.parent.healthBar:GetStatusBarColor();
	end	

	namePlate.healthBar:SetStatusBarColor(red, green, blue, 1);
end

function NamePlate:UpdateBorderColour(namePlate)
	local class = namePlate.class;
	local unit = namePlate.unit;
	local reaction = namePlate.reaction;
	local red, green, blue;
	local colourMode = ArenaLiveCore:GetDBEntry(addonName, "NamePlates/BorderColourMode");
	local usedColour;
	local _, instanceType = IsInInstance();
	
	if ( colourMode == "class" and ( class or reaction == "Hostile-Class" ) ) then
		red, green, blue = NamePlate:GetClassColour(namePlate.parent, class, reaction);
		usedColour = "CLASS";
	elseif ( colourMode == "team" and IsAddOnLoaded("ArenaLiveSpectator") ) then	
		red, green, blue = NamePlate:GetTeamColour(unit, reaction);
		usedColour = "TEAM";
		
		if ( not red ) then
			red, green, blue = NamePlate:GetReactionColour(reaction);
			usedColour = "REACTION";
		end	
	else
		red, green, blue = NamePlate:GetReactionColour(reaction);
		usedColour = "REACTION";
	end	
	
	--print("usedColour = "..tostring(usedColour).."; reaction = "..tostring(reaction));
	--if ( usedColour == "REACTION" and ( reaction == "Friendly" or reaction == "PvP-Friendly" ) ) then
	--	namePlate.border:SetTexture([[Interface\Addons\ArenaLiveNamePlates\Textures\NamePlateBorderUncoloured]]);
	--	namePlate.border:SetVertexColor(1, 1, 1, 1);
	--else
		namePlate.border:SetTexture([[Interface\Addons\ArenaLiveNamePlates\Textures\NamePlateBorderColoured]]);
		namePlate.border:SetVertexColor(red, green, blue, 1);		
	--end
	
	
end

-- TODO: Split styling function in different ones to make it a bit less messy:
function NamePlate:StyleNamePlate(namePlate)
	NamePlate:UpdateClassIcon(namePlate);
	NamePlate:UpdateHealthbarColour(namePlate);
	NamePlate:UpdateBorderColour(namePlate);
end

function NamePlate:GetReactionType(r, g, b)
	if ( r > 0.9 and g > 0.9 and b == 0 ) then
		return "Neutral";
	elseif ( r > 0.9 and g == 0 and b == 0 ) then
		return "Hostile";
	elseif ( g > 0.9 and r == 0 and b == 0 ) then
		return "PvP-Friendly";
	elseif ( b > 0.9 and r == 0 and g == 0 ) then
		return "Friendly";
	else
		return "Hostile-Class" -- Only hostile players can have class colours.
	end
end

function NamePlate:ReactionIsReaction(unitReaction, unitIsPlayer, plateReaction)
	--print("unitReaction:"..tostring(unitReaction).."; unitIsPlayer: "..tostring(unitIsPlayer).."; plateReaction: "..tostring(plateReaction));
	
	-- Class colour indication must be active in order to find enemy player. Otherways Mirror images etc. would be shown as players instead of NPCs:
	if ( unitReaction == "Hostile" and plateReaction == "Hostile" ) then
		return false;
	elseif ( UnitIsGhost("player") and unitIsPlayer and unitReaction == "Neutral" and ( plateReaction == "Hostile" or plateReaction == "Hostile-Class" ) ) then
		return true;
	elseif ( unitIsPlayer and unitReaction == "PvP-Friendly" and plateReaction == "Friendly" and IsInGroup() ) then
		return true;
	elseif ( unitIsPlayer and unitReaction == "Hostile" and plateReaction == "Hostile-Class" ) then
		return true;	
	elseif ( unitReaction == plateReaction ) then
		return true;
	end
	
	return false;

end

function NamePlate:UpdateAll()
	for _, newPlate in pairs(NamePlate.NamePlates) do
		if ( newPlate:IsVisible() ) then
			NamePlate_Update(newPlate);
		end
	end
end

function NamePlate:UpdateUnitCache(group, unit, state, updatePlates)

	local name;
	if ( group == "GROUP_ROSTER_UPDATE" ) then
	for i = 1, MAX_RAID_MEMBERS do
		if ( i <= MAX_PARTY_MEMBERS ) then
			unit = "party"..i;
			
			if ( UnitExists(unit)) then
				local oldName = NamePlate.UnitCache[unit];
				name = UnitName(unit);
				NamePlate.UnitCache[unit] = name;
				NamePlate.NameCache[name] = unit;
					
				if ( oldName ~= name and NamePlate.NameCache[oldName] == unit ) then
					NamePlate.NameCache[name] = nil;
				end
			else
				local oldName = NamePlate.UnitCache[unit];
				NamePlate.UnitCache[unit] = nil;
				
				if ( oldName and NamePlate.NameCache[oldName] == unit ) then
					NamePlate.NameCache[oldName] = nil;
				end
			end
		end
	
		unit = "raid"..i;
		if ( UnitExists(unit)) then
			local oldName = NamePlate.UnitCache[unit];
			name = UnitName(unit);
			NamePlate.UnitCache[unit] = name;
			NamePlate.NameCache[name] = unit;
					
			if ( oldName ~= name and NamePlate.NameCache[oldName] == unit ) then
				NamePlate.NameCache[name] = nil;
			end
		elseif ( NamePlate.UnitCache[unit] ) then
			local oldName = NamePlate.UnitCache[unit];
			NamePlate.UnitCache[unit] = nil;
				
			if ( oldName and NamePlate.NameCache[oldName] == unit ) then
				NamePlate.NameCache[oldName] = nil;
			end
		elseif ( i > MAX_PARTY_MEMBERS ) then
			break;
		end		
	end
	elseif ( group == "ARENA_OPPONENT_UPDATE" ) then
		if ( unit and state ) then
			
			if ( state == "seen" or state == "destroyed" or state == "cleared" ) then
				if ( UnitExists(unit)) then
					local oldName = NamePlate.UnitCache[unit];
					name = UnitName(unit);
					NamePlate.UnitCache[unit] = name;
					NamePlate.NameCache[name] = unit;
					
					if ( oldName ~= name and NamePlate.NameCache[oldName] == unit ) then
						NamePlate.NameCache[name] = nil;
					end							
				else
					NamePlate.UnitCache[unit] = nil;
					
					if ( oldName and NamePlate.NameCache[oldName] == unit ) then
						NamePlate.NameCache[oldName] = nil;
					end
				end		
			end
		else
			for i = 1, MAX_ARENA_OPPONENTS do
				NamePlate:UpdateUnitCache("ARENA_OPPONENT_UPDATE", "arena"..i, "seen", false);
			end
		end
	elseif ( group == "UPDATE_ALL" ) then
		NamePlate:UpdateUnitCache("GROUP_ROSTER_UPDATE", nil, nil, false);
		NamePlate:UpdateUnitCache("ARENA_OPPONENT_UPDATE", nil, nil, false);		
	end

	if ( updatePlates ) then
		NamePlate:UpdateAll();
	end
end

function NamePlate:OnEvent(event, ...)
	local unit, state, name;
	
	if ( event == "GROUP_ROSTER_UPDATE" ) then
		NamePlate:UpdateUnitCache("GROUP_ROSTER_UPDATE", nil, nil, true);
	elseif ( event == "ARENA_OPPONENT_UPDATE" ) then
		unit, state = ...;
		NamePlate:UpdateUnitCache("ARENA_OPPONENT_UPDATE", unit, state, true);
	elseif ( event == "ADDON_LOADED" and select(1, ...) == addonName ) then
		if ( not ArenaLiveCore:DatabaseExists(addonName) ) then
			ArenaLiveCore:AddDatabase(addonName, DEFAULT_SETTINGS, false);
		else
			ArenaLiveCore:UpdateDatabase(addonName, DEFAULT_SETTINGS);
		end
	end
end

-- Create a watch frame that will update the tables and create name plates when needed:
NamePlate.watchFrame = CreateFrame("Frame");

-- This function will check number of name plates etc:
local function OnUpdate(self, elapsed)
	
	-- New Child found, update Nameplates:
	if ( NamePlate.NumWorldFrameChildren < WorldFrame:GetNumChildren() ) then
		
		for _, frame in pairs({WorldFrame:GetChildren()}) do
			local frameName = frame:GetName();
			
			-- Check if frame is a nameplate:
			if ( frameName and string.find(frameName, "^NamePlate(%d*)$" ) ) then
				if ( not NamePlate.NamePlates[frame] ) then
					NamePlate:SetNamePlateStructure(frame);
					NamePlate:CreateNamePlate(frame);
				end
			end
		end
		
		NamePlate.NumWorldFrameChildren = WorldFrame:GetNumChildren();
	end

end
NamePlate.watchFrame:SetScript("OnUpdate", OnUpdate);

--[[ I created this function to get the frame structure of a nameplate, so that I know where to get which information from.
	 I started with just calling for the plate's children and expanded the function step by step for every bit of new information I got.
	 
	The end result of the function is the following data:
		* WorldFrame is the parent to all NamePlates.
		* A NamePlate uses no scripts.
		* A NamePlate contains 0 table entries (except for [0] = userdata of course).
		* A NamePlate has 2 UNNAMED children of the ObjectType "Frame".
		* A NamePlate has 0 direct regions.
		* Every time a NamePlate was hidden, it can be reused to show a different unit than before.
		
		* The first child contains:
			- 2 UNNAMED children of the Object Type "StatusBar":
				- StatusBar 1 is the HealthBar.
				- StatusBar 2 is the CastBar.
			- 7 UNNAMED regions (1, 2, 3, 5, 6, 7 = "Texture"; 4 = "FontString"):
				- Region 1 contains the texture: Interface\TargetingFrame\UI-TargetingFrame-Flash.
				- Region 2 contains the texture: Interface\Tooltips\NamePlate-Border.
				- Region 3 contains the texture: Interface\Tooltips\NamePlate-Glow.
				- Region 4 containts the text: <PLAYERLEVEL>
				- Region 5 contains the texture: Interface\TargetingFrame\UI-TargetingFrame-Skull.
				- Region 6 contains the texture: Interface\TargetingFrame\UI-RaidTargetingIcons.
				- Region 7 contains the texture: Interface\Tooltips\EliteNameplateIcon.
		
		* The second child contains:
			- 0 UNNAMED children.
			- 1 UNNAMED region of the type "Fonstring"
				- Region 1 contains the text: <PLAYERNAME>
]]--
local children;
function NamePlate:CrawlNamePlateData(nameplate)

	local children = {nameplate:GetChildren()};
	local regions = {nameplate:GetRegions()};
	local i = 1;
	
	print(nameplate:GetParent():GetName());
	for key, value in pairs(nameplate) do
		print(tostring(key).." = "..tostring(value));
	end
	
	for _, child in ipairs(children) do
		local frameType = child:GetObjectType();
		local subChildren = {child:GetChildren()}
		local subRegions = {child:GetRegions()};
		
		print(tostring("Child"..tostring(i))..": FrameType = "..tostring(frameType));
		
		local subID = 1;
		for _, subChild in ipairs(subChildren) do
			local minvalue, maxvalue, value;
			local objectType = subChild:GetObjectType();
			if ( objectType == "StatusBar" ) then
				minvalue, maxvalue = subChild:GetMinMaxValues();
				value = subChild:GetValue();
				local name = subChild:GetName();
				local subsubchildren = subChild:GetNumChildren();
				local subsubRegions = {subChild:GetRegions()};
				print("     "..tostring("SubChild"..tostring(subID))..": FrameType = "..tostring(objectType).."; MinValue = "..tostring(minvalue).."; MaxValue = "..tostring(maxvalue).."; Value = "..tostring(value)..";");
				subID = subID + 1; 
				for _, subsubRegion in pairs(subsubRegions) do
					local subsubRegionType = subsubRegion:GetObjectType();
					if ( subsubRegionType == "FontString" ) then
						local subsubText = subsubRegion:GetText();
						print("          5:"..tostring(subsubText));
					end
				end
			end
		end
		
		local subRegionID = 1;
		for _, region in ipairs(subRegions) do
			local regionType = region:GetObjectType();
			local content;
			
			if ( regionType == "Texture" ) then
				content = region:GetTexture();
			elseif ( regionType == "FontString" ) then
				content = region:GetText();
			end
			
			print("     "..tostring("SubRegion")..tostring(subRegionID)..": RegionType = "..tostring(regionType).."; Content = "..tostring(content));
			subRegionID = subRegionID + 1;
		end
		
		i = i + 1;
	end
	
	local regionID = 1;
	for _, region in ipairs(regions) do
		local regionName = region:GetName();
		local regionType = region:GetObjectType();
		print (tostring(regionName).."(Region"..tostring(subRegionID).."): RegionType = "..tostring(regionType));
		regionID = regionID + 1;
	end
end

local Scripts =
	{
		["OnShow"] = true,
		["OnHide"] = true,
		["OnLoad"] = true,
		["OnEvent"] = true,
		["OnUpdate"] = true,
		["OnEnter"] = true,
		["OnLeave"] = true,
	}
	
function NamePlate:ShowNamePlateScripts(nameplate)
	local mainFrame, nameFrame = nameplate:GetChildren();
	local healthBar, castBar = mainFrame:GetChildren();
	
	for script, value in pairs(Scripts) do
		print(script..": "..tostring(healthBar:GetScript(script)));
	end

end

function NamePlate:PrintTables()
	print ("NamePlate.NameCache:")
	for name, unit in pairs(NamePlate.NameCache) do
		print("     "..name.." = "..unit);
	end

	print ("NamePlate.UnitCache:")
	for unit, name in pairs(NamePlate.UnitCache) do
		print("     "..unit.." = "..name);
	end	
	
end